//
//  ViewController.m
//  WaterfallToTableTransition
//
//  Created by Noval Agung Prayogo on 6/25/13.
//  Copyright (c) 2013 Noval Agung Prayogo. All rights reserved.
//

#import "ViewController.h"

@interface ViewController ()

@end

@implementation ViewController

@synthesize containerView, collectionView, tableView, data, dataColor, collectionData, tableData, collectionViewClone, tableViewClone;

int randomLimit(int limit) {
    int divisor = RAND_MAX / (limit + 1);
    int retval;
    
    do {
        retval = rand() / divisor;
    } while (retval > limit);
    
    return retval;
}

# pragma mark implicit declaration

- (void)viewDidLoad {
    [super viewDidLoad];
    [self prepareLayout];
    [self prepareData];
    [self prepareCollectionView];
    [self prepareTableView];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

# pragma mark prepare 

- (void)prepareLayout {
    containerView.clipsToBounds = YES;
}

- (void)prepareData {
    data = [[NSMutableArray alloc] initWithCapacity:30];
    dataColor = [[NSMutableArray alloc] initWithCapacity:30];
    for (int i = 0; i < 30; i++) {
        [data insertObject:[NSNumber numberWithInt:100 + randomLimit(6) * 10 + randomLimit(10)] atIndex:i];
        [dataColor insertObject:[UIColor random] atIndex:i];
    }
}

- (void)prepareCollectionView {
    UICollectionViewWaterfallLayout *layout = (UICollectionViewWaterfallLayout *)collectionView.collectionViewLayout;
    layout.columnCount = 5;
    layout.itemWidth = 100;
    [collectionView registerClass:[UICollectionViewCell class] forCellWithReuseIdentifier:@"collectionCell"];
    [collectionView reloadData];
}

- (void)prepareTableView {
    [tableView registerClass:[UITableViewCell class] forCellReuseIdentifier:@"tableCell"];
    tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
}

# pragma mark collection delegate - datasource

- (UICollectionViewCell *)collectionView:(UICollectionView *)c cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    
    UICollectionViewCell* cell = [c dequeueReusableCellWithReuseIdentifier:@"collectionCell" forIndexPath:indexPath];
    if (cell == nil) cell = [[UICollectionViewCell alloc] initWithFrame:CGRectZero];
    
    for (id i in cell.subviews) [i removeFromSuperview];
    
    cell.backgroundColor = [dataColor objectAtIndex:indexPath.row];
    
    return cell;
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return [data count];
}

- (CGFloat)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewWaterfallLayout *)collectionViewLayout heightForItemAtIndexPath:(NSIndexPath *)indexPath {
    return [[data objectAtIndex:indexPath.row] floatValue];
}

# pragma mark tableview delegate - datasource

- (UITableViewCell *)tableView:(UITableView *)t cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [t dequeueReusableCellWithIdentifier:@"tableCell" forIndexPath:indexPath];
    if (cell == nil) cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"tableCell"];
    
    for (id i in cell.subviews) [i removeFromSuperview];
    
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    cell.layer.backgroundColor = [(UIColor *)[dataColor objectAtIndex:indexPath.row] CGColor];
    
    return cell;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [data count];
}

# pragma mark here we go !

- (IBAction)startTransition:(UIButton *)sender {
    [self toThumb];
}


- (void)prepareAnimation {
    for (id i in collectionViewClone.subviews) [i removeFromSuperview];
    for (id i in tableViewClone.subviews) [i removeFromSuperview];
    
    [collectionViewClone removeFromSuperview];
    [tableViewClone removeFromSuperview];
    
    collectionViewClone = nil;
    tableViewClone = nil;
}


- (void)toThumb {
    [self prepareAnimation];
    
    collectionData = [[NSMutableArray alloc] init];
    tableViewClone = [[UIView alloc] initWithFrame:CGRectMake(0, 0, tableView.frame.size.width, [tableView superview].frame.size.height)];
    tableViewClone.backgroundColor = [UIColor clearColor];
    tableViewClone.clipsToBounds = NO;
    
    NSIndexPath *resetTableIndexPath = [[tableView indexPathsForVisibleRows] objectAtIndex:0];
    [tableView scrollToRowAtIndexPath:resetTableIndexPath atScrollPosition:UITableViewScrollPositionTop animated:NO];
    int collectionShouldSection = resetTableIndexPath.row - (resetTableIndexPath.row % 5);
    NSIndexPath *resetCollectionIndexPath = [NSIndexPath indexPathForItem:collectionShouldSection + 4 inSection:0];
    [collectionView performBatchUpdates:^{
        [collectionView scrollToItemAtIndexPath:resetCollectionIndexPath atScrollPosition:UICollectionViewScrollPositionTop animated:NO];
    } completion:^(BOOL finished) {
        
        int collectionFirstIndexPathRow = 0;
        int collectionLastIndexPathRow = 0;
        int tableTopPadding = tableView.contentOffset.y;
        int collectionTopPadding = collectionView.contentOffset.y;
        
        /** reorder collection indexpath */ {
            NSArray *visibleCells = [collectionView indexPathsForVisibleItems];
            for (int i = 0; i < [[collectionView visibleCells] count]; i++) {
                int row = [[visibleCells objectAtIndex:i] row];
                if (i == 0) {
                    collectionFirstIndexPathRow = collectionLastIndexPathRow = row;
                } else if ([[visibleCells objectAtIndex:i] row] < collectionFirstIndexPathRow) {
                    collectionFirstIndexPathRow = row;
                } else if ([[visibleCells objectAtIndex:i] row] > collectionLastIndexPathRow) {
                    collectionLastIndexPathRow = row;
                }
            }
        }
        
        for (int i = collectionFirstIndexPathRow; i - 1 < collectionLastIndexPathRow; i++) {
            /** get destination rect */ {
                NSIndexPath *indexPath = [NSIndexPath indexPathForItem:i inSection:0];
                [collectionView scrollToItemAtIndexPath:indexPath atScrollPosition:UICollectionViewScrollPositionTop animated:NO];
                UICollectionViewCell *cell = [collectionView cellForItemAtIndexPath:indexPath];
                UIImageView *imageView = [[UIImageView alloc] initWithFrame:cell.frame];
                imageView.image = [UIImage imageWithCGImage:CGImageCreateWithImageInRect([collectionView screenshotWithOptimization:YES].CGImage, tableView.frame)];
                int y = imageView.frame.origin.y - collectionTopPadding;
                imageView.frame = CGRectMake(imageView.frame.origin.x + collectionView.frame.origin.x, y, imageView.frame.size.width, imageView.frame.size.height);
                [collectionData addObject:[NSValue valueWithCGRect:imageView.frame]];
            }
            
            /** get departure image */ {
                NSIndexPath *indexPath = [NSIndexPath indexPathForItem:i inSection:0];
                [tableView scrollToRowAtIndexPath:indexPath atScrollPosition:UITableViewScrollPositionTop animated:NO];
                UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
                UIImageView *imageView = [[UIImageView alloc] initWithFrame:cell.frame];
                cell.frame = CGRectMake(0, 0, cell.frame.size.width, cell.frame.size.height);
                imageView.image = [UIImage imageWithCGImage:CGImageCreateWithImageInRect([cell screenshotWithOptimization:YES].CGImage, cell.frame)];
                int y = ((i - collectionFirstIndexPathRow) * cell.frame.size.height);
                y = imageView.frame.origin.y - tableTopPadding;
                imageView.frame = CGRectMake(imageView.frame.origin.x, y, imageView.frame.size.width, cell.frame.size.height);
                imageView.backgroundColor = [UIColor clearColor];
                [tableViewClone addSubview:imageView];
            }
        }
        
        [collectionView scrollToItemAtIndexPath:resetCollectionIndexPath atScrollPosition:UICollectionViewScrollPositionTop animated:NO];
        [tableView scrollToRowAtIndexPath:resetTableIndexPath atScrollPosition:UITableViewScrollPositionTop animated:NO];
        
        NSLog(@"%f", tableView.contentOffset.y);
        NSLog(@"%@", collectionData);
        NSLog(@"%@", tableViewClone.subviews);
        
        [containerView addSubview:tableViewClone];
        tableView.hidden = YES;
        collectionView.hidden = YES;
        [UIView animateWithDuration:1. animations:^{
            for (int i = 0; i < [tableViewClone.subviews count]; i++) {
                [[tableViewClone.subviews objectAtIndex:i] setFrame:[[collectionData objectAtIndex:i] CGRectValue]];
            }
        } completion:^(BOOL finished) {
            [tableViewClone removeFromSuperview];
            tableView.hidden = NO;
            collectionView.hidden = NO;
        }];
    }];
}

@end
